#!/usr/bin/python

# This program controls a "corretto" style coffee roaster
# using a R-Pi with a Pi-Face interface board
#
# It outputs a PWM value to a file that is read regularly by a
# background-running program that drives the heat gun output. 
#
# In addition it runs a few LEDs to indicate the state of the roasting process.
# It needs a button-push to progress through the stages from
# Ready -> Preheat -> Roast -> End -> repeat
#
# At intervals, the program checks the temperature and adjusts the
# heater settings. At the same time (during ROAST) it outputs the
# time/temp/target to a file for viewing via a web page.
#
# The temp is sensed by a Yocto-Thermocouple USB board, which is
# interrogated via a background-running program.

import time
import piface.pfio as pfio
import datetime
import os,sys
sys.path.append(os.path.join("/home/pi/yocto/Sources"))
from yocto_api import *
from yocto_temperature import *

pfio.init()


# =========================
# Configuration
# 
# Where the profile file is located
PROFILE = "/home/pi/roaster/profile.txt"
# Where the output files are stored
WORKDIR = "/home/pi/roaster"
# How often to update the roast temps (seconds)
SLEEPTIME = 5
# number of update cycles to look ahead
LOOKAHEAD = 14
# Where the pulse width mod file is located
PWMFILE = "pwm.txt"
# Where the current bean temp file is located
BEANTEMPFILE = "beantemp.txt"
# Where the K1 log file is located
K1FILE = "K1.txt"

PWMMAX = 100.0
AMBIENT = 12.0
ALPHA = 0.5


# =========================
# Set up the I/O numbers for the PiFace board
# Inputs
IBUTTON = 0
IKILL = 1
# Outputs
OHEAT = 7
OREADY = 6
OPREHEAT = 5
OROAST = 4
OEND = 3

# Set the thermocouple module info
sensor= YTemperature.FindTemperature('THRMCPL1-189C5.temperature1')

# ================
# Program starts
#

os.chdir(WORKDIR)

# RESET outputs and stop heating
pfio.digital_write(OREADY,0)
pfio.digital_write(OPREHEAT,0)
pfio.digital_write(OROAST,0)
pfio.digital_write(OEND,0)
OUTPUTFILE = open(PWMFILE,'w')
OUTPUTFILE.write("0")
OUTPUTFILE.close()

# Start the PWM
os.system("/usr/bin/python /home/pi/roaster/roaster-pwm.py &")

# Register the Yocto Thermocouple
errmsg=YRefParam()

if YAPI.RegisterHub("usb", errmsg)!= YAPI.SUCCESS:
	while (True):
		pfio.digital_write(OEND,0)
		time.sleep(0.5)
		pfio.digital_write(OEND,1)
		time.sleep(0.5)

# check it is on line
if not(sensor.isOnline()):
	while (True):
		pfio.digital_write(OEND,0)
		time.sleep(0.5)
		pfio.digital_write(OEND,1)
		time.sleep(0.5)


# set initial value 
BEANTEMP = 25

# This checks that the button has been released before the next input
BTNRESET = 1


# Main Loop starts
while (True):


##############################################################		
# READY loop starts
	while (True):

# Turn on READY lamp
		pfio.digital_write(OREADY,1)
	
# Check for KILL button and exit with all lamps on
	
		BTNPRESS = pfio.digital_read(IKILL)
		if BTNPRESS == 1:
			pfio.digital_write(OPREHEAT,1)
			pfio.digital_write(OROAST,1)
			pfio.digital_write(OEND,1)
			exit()

# Check for button press and reset if un-pressed
		BTNPRESS = pfio.digital_read(IBUTTON)
		if  BTNPRESS == 1:
			if BTNRESET == 1:
				pfio.digital_write(OREADY,0)
				BTNRESET = 0
				break
		else:
			BTNRESET = 1

# Update the screen
		os.system('clear')
		print chr(27)+"[32m"+"READY \n"
		while (True):
			if (sensor.isOnline()):
				NEWTEMP = sensor.get_currentValue()
				BEANTEMP = BEANTEMP * ( 1- ALPHA ) + NEWTEMP * ALPHA
				BEANTEMP = float("%2.0f" % BEANTEMP)

				break
		print chr(27)+"[37m"+"Current temp:", '%.0f' % BEANTEMP

		time.sleep(0.5)
# end of READY loop

##############################################################	
# PREHEAT Starts

# save last temp as ambient

	AMBIENT = BEANTEMP

# Update the screen
	os.system('clear')
	print chr(27)+"[33m"+"PREHEAT \n"
	print chr(27)+"[37m"+"Current temp:"
	print "Target temp :"
	print "Power       :"


# Turn on PREHEAT lamp
	pfio.digital_write(OPREHEAT,1)

# Save the time 
	LASTLOG = time.time()

# Update the Smoothing coefficient
	F = open("/var/www/control/alpha.txt","r")
	ALPHA = float(F.read())
	F.close()

#Preheat Power
	F = open("/var/www/control/prepower.txt","r")
	PREPOWER = F.read()
	F.close()

# Preheat temp
	F = open("/var/www/control/pretemp.txt","r")
	PRETEMP = float(F.read())
	F.close()


# PREHEAT loop
	while (True):

# Check for button press and reset if un-pressed
		BTNPRESS = pfio.digital_read(IBUTTON)
		if  BTNPRESS == 1:
			if BTNRESET == 1:
				pfio.digital_write(OPREHEAT,0)
				BTNRESET = 0
				break
		else:
			BTNRESET = 1

# Check Bean Temp after each sleeptime and turn on/off heater
		NOW = time.time()
		if ( NOW - LASTLOG ) > SLEEPTIME: 
			LASTLOG = time.time()

# Update the temp here and blink lamp
			pfio.digital_write(OPREHEAT,0)
			while (True):
				if (sensor.isOnline()):
					NEWTEMP = float("%2.1f" % sensor.get_currentValue())
					break

			if NEWTEMP > 0:
				BEANTEMP = BEANTEMP * ( 1- ALPHA ) + NEWTEMP * ALPHA
				BEANTEMP = float("%2.1f" % BEANTEMP)

			time.sleep(0.1)
			pfio.digital_write(OPREHEAT,1)

# Update the screen
			os.system('clear')
			print chr(27)+"[33m"+"PREHEAT \n"
			print chr(27)+"[37m"+"Current temp:", '%.0f' % BEANTEMP, "C"
			print "Target temp :", '%.0f' % PRETEMP, "C"


# Turn the heater on or off
			OUTPUTFILE = open(PWMFILE,'w')
			if PRETEMP > BEANTEMP:
				OUTPUTFILE.write(PREPOWER)
				print "Power       :", '%.0f' % float(PREPOWER), "%"
			
			else:
				OUTPUTFILE.write("30")
				print "Power       :", "30", "%"
		
			OUTPUTFILE.close()



		time.sleep(0.1)
# end of PREHEAT loop

##############################################################	
# ROAST STARTS

# Turn on ROAST lamp
	pfio.digital_write(OROAST,1)

# Update the screen
	os.system('clear')
	print chr(27)+"[31m"+"ROAST"
	print chr(27)+"[37m"+"Elapsed time:"
	print "Current temp:"
	print "Target temp :"
	print "Power       :"

# Set the lookahead parameters 
# k1 ws 200, k2 was 0.28
	F = open("/var/www/control/k1.txt","r")
	K1 = float(F.read())
	F.close()
	F = open("/var/www/control/k2.txt","r")
	K2 = float(F.read())
	F.close()

# Bean Mass
	F = open("/var/www/control/beanmass.txt","r")
	BEANMASS = float(F.read())
	F.close()

# Adjust K1 for variable bean mass (calibrated at 300g)
	K1 = K1 * BEANMASS/300

#Roast start Power/time
	F = open("/var/www/control/rstartpower.txt","r")
	RSTARTPOWER = float(F.read())
	F.close()

	F = open("/var/www/control/rstarttime.txt","r")
	RSTARTTIME = float(F.read())
	F.close()

# Max roast temp 
	F = open("/var/www/control/peaktemp.txt","r")
	PEAKTEMP = float(F.read())
	F.close()

# Max roast time 
	F = open("/var/www/control/roasttime.txt","r")
	ROASTTIME = float(F.read())
	F.close()

# Calculate the temp/time conversion factors to allow for variable roast profiles

	TEMPF = PEAKTEMP/215
	TIMEF = ROASTTIME/20


# Set up the log files
	START = time.time()
	STARTTIME=time.strftime("%c")

	OUTPUTFILE = open("roaster.txt",'w')
	OUTPUTFILE.write(STARTTIME + ". Ambient: " + str('%.0f' % AMBIENT) + "C\n")
	OUTPUTFILE.close()

# ROAST loop
	while (True):


# Check for button press and reset if un-pressed
		BTNPRESS = pfio.digital_read(IBUTTON)
		if  BTNPRESS == 1:
			if BTNRESET == 1:
				pfio.digital_write(OROAST,0)
				BTNRESET = 0
				break
		else:
			BTNRESET = 1

# Check Bean Temp after each sleeptime and send PWM value to heater program
		NOW = time.time()
		if ( NOW - LASTLOG ) > SLEEPTIME: 
			LASTLOG = time.time()

# Update the temp here and blink lamp
			pfio.digital_write(OROAST,0)
			while (True):
				if (sensor.isOnline()):
					NEWTEMP = float("%2.1f" % sensor.get_currentValue())
					break

# Smoothing the temperature reading

			BEANTEMP = BEANTEMP * ( 1- ALPHA ) + NEWTEMP * ALPHA
			BEANTEMP = float("%2.1f" % BEANTEMP)

			time.sleep(0.1)
			pfio.digital_write(OROAST,1)

			RUNNING = NOW - START 

# Check the profile and update the target temp
			INPUTFILE = open(PROFILE,"r")
			DATA = INPUTFILE.readlines()
			for LINE in DATA:
				PTIME, PTEMP = LINE.split(",")
# Process the raw inputs
				NEWTIME = float(PTIME) * 60 * TIMEF
				NEWTEMP = float(PTEMP) * TEMPF

				if NEWTEMP == 0:
					break

				if RUNNING > NEWTIME:
					LASTTIME = NEWTIME
					LASTTEMP = NEWTEMP
				else:
					NEXTTIME = NEWTIME
					NEXTTEMP = NEWTEMP
					break

			INPUTFILE.close()

# Check if exit from target temp loop is due to eof
			if NEWTEMP == 0:
				break
# Interpolate the new target temp
			TARGET = LASTTEMP + ( NEXTTEMP - LASTTEMP ) * ( RUNNING - LASTTIME ) / ( NEXTTIME - LASTTIME )
			TARGET = float("%2.1f" % TARGET)

# Now check the profile for the Lookahead target

			LATIME = RUNNING + LOOKAHEAD * SLEEPTIME
			INPUTFILE = open(PROFILE,"r")
			DATA = INPUTFILE.readlines()
			for LINE in DATA:
				PTIME, PTEMP = LINE.split(",")
# Process the raw inputs
				NEWTIME = float(PTIME) * 60 * TIMEF
				NEWTEMP = float(PTEMP) * TEMPF

				if NEWTEMP == 0:
					break

				if LATIME > NEWTIME:
					LASTTIME = NEWTIME
					LASTTEMP = NEWTEMP
				else:
					NEXTTIME = NEWTIME
					NEXTTEMP = NEWTEMP
					break

			INPUTFILE.close()

# Check if exit from target temp loop is due to eof
			if NEWTEMP == 0:
				break
# Interpolate the new lookahead target temp

			LATARGET = LASTTEMP + ( NEXTTEMP - LASTTEMP ) * ( LATIME - LASTTIME ) / ( NEXTTIME - LASTTIME )
			LATARGET = float("%2.1f" % LATARGET)



# Calculate the PWM
			PWM = K1 * ( LATARGET - BEANTEMP ) / ( LOOKAHEAD * SLEEPTIME ) + K2 * ( BEANTEMP - AMBIENT )

			PWM = float("%4.1f" % PWM)

			if PWM > PWMMAX:
				PWM = PWMMAX

			if PWM < 0:
				PWM = 0	

# special heat profile just after startup
			if RUNNING < RSTARTTIME :
				PWM = RSTARTPOWER

# Update the PWM file
			OUTPUTFILE = open(PWMFILE,'w')
			OUTPUTFILE.write(str(PWM))
			OUTPUTFILE.close()
			

#		Update the log file
			MINS = '%.2f' % ( RUNNING / 60 )

			OUTPUTFILE = open("roaster.txt",'a')
			OUTPUTFILE.write(MINS + "," + str(BEANTEMP) + "," + str(TARGET) + "," + str(PWM) + "\n")
			OUTPUTFILE.close()

# Update the screen
			os.system('clear')
			print chr(27)+"[31m"+"ROAST"
			print chr(27)+"[37m"+"Elapsed time:", '%.1f' % float(MINS), "m"
			print "Current temp:", '%.0f' % BEANTEMP, "C"
			print "Target temp :", '%.0f' % TARGET, "C"
			print "Power       :", '%.0f' % PWM, "%"


# End of update loop
		
		time.sleep(0.1)	
# End of ROAST loop
		
##############################################################	
# END starts


# Turn off heater, and turn on END lamp
	OUTPUTFILE = open(PWMFILE,'w')
	OUTPUTFILE.write("0")
	OUTPUTFILE.close()
	pfio.digital_write(OROAST,0)
	pfio.digital_write(OEND,1)

# Update the screen
	os.system('clear')
	print "END"



# END loop
	while(True):

# Check for button press and reset if un-pressed
		BTNPRESS = pfio.digital_read(IBUTTON)
		if  BTNPRESS == 1:
			if BTNRESET == 1:
				pfio.digital_write(OEND,0)
				BTNRESET = 0
				break
		else:
			BTNRESET = 1
		
# Update the screen
                os.system('clear')
                print "END \n"
                while (True):
                        if (sensor.isOnline()):
                                NEWTEMP = sensor.get_currentValue()
                                BEANTEMP = BEANTEMP * ( 1- ALPHA ) + NEWTEMP * ALPHA
                                BEANTEMP = float("%2.0f" % BEANTEMP)

                                break
                print "Current temp:", '%.0f' % BEANTEMP

                time.sleep(0.5)


# End of END LOOP

# End of main loop

